#include "conversion.h"
#include "yv12toyuy2.h"
#include "yuy2toyv12.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

static void syntax()
{
   fprintf( stderr, "syntax : conversion out in width height type yv12type [interlaced]\n" );

   fprintf( stderr, " * in / out       : file names\n" );
   fprintf( stderr, " * width / height : integers\n" );
   fprintf( stderr, " * type           : yv12 or yuy2\n" );
   fprintf( stderr, " * yv12type       : 0, 1 or 2\n" );
   fprintf( stderr, " * interlaced     : if present, process as interlaced\n" );
}

static yuy2toyv12_f *yuy2toyv12_f_array[2][3] = 
{
   {
      yuy2toyv12_frame_1,
      yuy2toyv12_frame_2,
      yuy2toyv12_frame_3,
   },
   {
      yuy2toyv12_field_1,
      yuy2toyv12_field_2,
      yuy2toyv12_field_3,
   },
};

static yuy2toyv12_f *yuy2toyv12_f_isse_array[2][3] = 
{
   {
      yuy2toyv12_frame_1_isse,
      yuy2toyv12_frame_2_isse,
      yuy2toyv12_frame_3_isse,
   },
   {
      yuy2toyv12_field_1_isse,
      yuy2toyv12_field_2_isse,
      yuy2toyv12_field_3_isse,
   },
};

static yv12toyuy2_f *yv12toyuy2_f_array[2][3] = 
{
   {
      yv12toyuy2_frame_1,
      yv12toyuy2_frame_2,
      yv12toyuy2_frame_3,
   },
   {
      yv12toyuy2_field_1,
      yv12toyuy2_field_2,
      yv12toyuy2_field_3,
   },
};

static yv12toyuy2_f *yv12toyuy2_f_isse_array[2][3] = 
{
   {
      yv12toyuy2_frame_1_isse,
      yv12toyuy2_frame_2_isse,
      yv12toyuy2_frame_3_isse,
   },
   {
      yv12toyuy2_field_1_isse,
      yv12toyuy2_field_2_isse,
      yv12toyuy2_field_3_isse,
   },
};


void main(int nArgc, char *pArgv[])
{
   FILE *in = NULL, *out = NULL;
   int nWidth, nHeight;
   bool isInputYV12;
   uint8_t *pIn[3], *pOut[3], *pTest[3];
   int nInputSize, nOutputSize;
   bool isInterlaced = false;
   int nYv12Csp;

   fprintf( stderr, "conversion : a raw yv12/yuy2 converter\n" );

   if ( nArgc < 7 || nArgc > 8 )
      syntax(), exit(-1);


   in = fopen( pArgv[2], "rb" );
   if ( !in )
      fprintf( stderr, "unable to open input file\n" ), exit(-1);

   out = fopen( pArgv[1], "wb" );
   if ( !out )
      fprintf( stderr, "unable to open output file\n" ), exit(-1);

   nWidth = atoi( pArgv[3] );
   nHeight = atoi( pArgv[4] );

   if ( nWidth <= 2 || nWidth > 4096 )
      fprintf( stderr, "invalid width(%i)\n", nWidth ), exit(-1);

   if ( nHeight <= 2 || nHeight > 4096 )
      fprintf( stderr, "invalid height(%i)\n", nHeight ), exit(-1);

   if ( !strncasecmp( pArgv[5], "yv12", 4 ) )
      isInputYV12 = true;
   else if ( !strncasecmp( pArgv[5], "yuy2", 4 ) )
      isInputYV12 = false;
   else
      syntax(), exit(-1);

   nYv12Csp = atoi( pArgv[6] );

   if ( nYv12Csp < 0 || nYv12Csp > 2 )
      syntax(), exit(-1);

   if ( nArgc == 8 )
   {
      if ( !strncasecmp( pArgv[7], "interlaced", 10 ) )
         isInterlaced = true;
      else
         syntax(), exit(-1);
   }

   fprintf( stderr, "\nConverting %s [%ix%i, %s%s] to %s[%ix%i, %s%s]\n",
      pArgv[2], nWidth, nHeight, isInputYV12 ? "yv12" : "yuy2", isInterlaced ? ", interlaced" : "",
      pArgv[1], nWidth, nHeight, isInputYV12 ? "yuy2" : "yv12", isInterlaced ? ", interlaced" : ""
      );

   /* allocates buffers */
   nInputSize = nWidth * nHeight * ( 4 + 2 * (isInputYV12 ? 1 : 2) ) / 4;
   nOutputSize = nWidth * nHeight * ( 4 + 2 * (isInputYV12 ? 2 : 1) ) / 4;

   pIn[0] = malloc( sizeof( uint8_t ) * nInputSize );
   pOut[0] = malloc( sizeof( uint8_t ) * nOutputSize );
   pTest[0] = malloc( sizeof( uint8_t ) * nOutputSize );

   if ( isInputYV12 )
   {
      pIn[1] = pIn[0] + nWidth * nHeight;
      pIn[2] = pIn[1] + nWidth * nHeight / 4;
   }
   else
   {
      pOut[1] = pOut[0] + nWidth * nHeight;
      pOut[2] = pOut[1] + nWidth * nHeight / 4;
      pTest[1] = pTest[0] + nWidth * nHeight;
      pTest[2] = pTest[1] + nWidth * nHeight / 4;
   }

   while ( !feof( in ) )
   {
      /* read one frame */
      fread( pIn[0], nInputSize, sizeof( uint8_t ), in );

      /* convert it */
      if ( isInputYV12 )
      {
         yv12toyuy2_f_isse_array[isInterlaced][nYv12Csp]( pOut[0], nWidth * 2, pIn[0], nWidth,
                                                          pIn[1], nWidth / 2, pIn[2], nWidth / 2,
                                                          nWidth, nHeight );
      }
      else
      {
         yuy2toyv12_f_isse_array[isInterlaced][nYv12Csp]( pOut[0], nWidth, pOut[1], nWidth / 2,
                                                          pOut[2], nWidth / 2, pIn[0], nWidth * 2,
                                                          nWidth, nHeight );
      }

      /* write it */
      fwrite( pOut[0], nOutputSize, sizeof( uint8_t ), out );
   }

   free( pIn[0] );
   free( pOut[0] );

   fclose( in );
   fclose( out );

   fprintf( stderr, "\nConversion succesfully done !\n" );

   exit(0);
}