#ifndef __Yuy2_To_Yv12_H__
#define __Yuy2_To_Yv12_H__

/* 

   YUY2 packed to YV12 planar colorspace conversions

   The classic pitfall of YV12 to YUY2 colorspace conversions
   is to forget to take into account where chroma pixels lie
   spatially, relatively to the luma pixels.

   Neither is that position isn't exactly defined. You can find 
   YV12 colorspaces that differ only by the position of their
   chroma pixels.

   That implies that several converters must be implemented. 
   Ideally, one converter for each possible pair of 
   ( input colorspace, output colorspace ). 

   Note also that we don't care about the horizontal position 
   of the chroma pixel, which we'll assume fixed and the same
   in both colorspace.

   Note the following convention : X will be a luma pixel, O a
   chroma one.

   Luckily, the chroma pixel, in YUY2 progressive, can only be on
   the luma pixel, so that case is easily solved

yuy2 progressive :

   X   O   X   O   X   O   X   O   X



   X   O   X   O   X   O   X   O   X



   X   O   X   O   X   O   X   O   X



   X   O   X   O   X   O   X   O   X
   

   Let's see now YV12 progressive : we have three possible positions :


yv12 progressive 1 ( most classic one ) : 

   X       X       X       X       X

       O       O       O       O

   X       X       X       X       X



   X       X       X       X       X

       O       O       O       O

   X       X       X       X       X

yv12 progressive 2 : 

   X   O   X   O   X   O   X   O   X



   X       X       X       X       X



   X   O   X   O   X   O   X   O   X



   X       X       X       X       X

yv12 progressive 3 : 

   X       X       X       X       X



   X   O   X   O   X   O   X   O   X



   X       X       X       X       X



   X   O   X   O   X   O   X   O   X

   Now, when we want to make a conversion from yuy2 to yv12, we
   must use the proper interpolation according to the final
   yv12 colorspace.

   In YUY2 ---> YV12 1, the bilinear kernel used will be (1, 3, 3, 1)
   In YUY2 ---> YV12 2, the bilinear kernel used will be (1, 2, 1, 0)
   In YUY2 ---> YV12 3, the bilinear kernel used will be (0, 1, 2, 1)

   Now, lets think a little about how interlace is stored in YV12.
   It's not as abvious as it seems. One would first think that chroma
   pixels lies in their respective field as if the field was itself
   progressive. But what happens most is actually chroma pixels been
   where they should be on the whole progressive frame.

   That implies that if we consider a field alone, chroma pixels are
   even more weirdly placed.

   Let's examined each YV12 colorspace successively : 

YV12 interlaced 1, top field :

   X       X       X       X       X 
       O       O       O       O     


   X       X       X       X       X 

                                     

   X       X       X       X       X 
       O       O       O       O     


   X       X       X       X       X 

YV12 interlaced 1, bottom field :

   X       X       X       X       X 

   
       O       O       O       O     
   X       X       X       X       X 

                                     

   X       X       X       X       X 

   
       O       O       O       O     
   X       X       X       X       X 

YV12 interlaced 2, top field :

   X   O   X   O   X   O   X   O   X



   X       X       X       X       X



   X   O   X   O   X   O   X   O   X



   X       X       X       X       X

YV12 interlaced 2, bottom field, and YV12 interlaced 3, top field :

   X       X       X       X       X

       O       O       O       O

   X       X       X       X       X



   X       X       X       X       X

       O       O       O       O

   X       X       X       X       X

YV12 interlaced 3, bottom field :

   X       X       X       X       X



   X   O   X   O   X   O   X   O   X



   X       X       X       X       X



   X   O   X   O   X   O   X   O   X

   That implies the following kernels :

   YV12 interlaced 1, top     ---> (3, 7, 5, 1)
   YV12 interlaced 1, bottom  ---> (1, 5, 7, 3)
   YV12 interlaced 2, top     ---> (1, 2, 1, 0)
   YV12 interlaced 2, bottom  ---> (1, 3, 3, 1)
   YV12 interlaced 3, top     ---> (1, 3, 3, 1)
   YV12 interlaced 3, bottom  ---> (0, 1, 2, 1)

*/

typedef void (yuy2toyv12_f)(uint8_t *pDstY, int nDstYPitch,
                            uint8_t *pDstU, int nDstUPitch,
                            uint8_t *pDstV, int nDstVPitch,
                            const uint8_t *pSrc, int nSrcPitch,
                            int nWidth, int nHeight);

yuy2toyv12_f yuy2toyv12_frame_1;
yuy2toyv12_f yuy2toyv12_frame_2;
yuy2toyv12_f yuy2toyv12_frame_3;
yuy2toyv12_f yuy2toyv12_field_1;
yuy2toyv12_f yuy2toyv12_field_2;
yuy2toyv12_f yuy2toyv12_field_3;

extern yuy2toyv12_f yuy2toyv12_frame_1_isse;
extern yuy2toyv12_f yuy2toyv12_frame_2_isse;
extern yuy2toyv12_f yuy2toyv12_frame_3_isse;
extern yuy2toyv12_f yuy2toyv12_field_1_isse;
extern yuy2toyv12_f yuy2toyv12_field_2_isse;
extern yuy2toyv12_f yuy2toyv12_field_3_isse;

#endif // __Yuy2_To_Yv12_H__
